@@ -276,7 +276,19 @@ fn make_url(
 async fn default(req: actix_web::HttpRequest) -> Result<HttpResponse, Error> {
   let mut is_proxy = false;
   // Try to construct a Gemini URL
-  let url = make_url(req.path(), false, &mut is_proxy).unwrap();
+  let url = make_url(
+    &format!("{}{}", req.path(), {
+      if !req.query_string().is_empty() || req.uri().to_string().ends_with('?')
+      {
+        format!("?{}", req.query_string())
+      } else {
+        "".to_string()
+      }
+    }),
+    false,
+    &mut is_proxy,
+  )
+  .unwrap();
   // Make a request to get Gemini content and time it.
   let mut timer = Instant::now();
   let mut response = match gmi::request::make_request(&url) {
@@ -301,25 +313,38 @@ async fn default(req: actix_web::HttpRequest) -> Result<HttpResponse, Error> {
   timer = Instant::now();
 
   // Convert Gemini Response to HTML and time it.
-  let mut html_context = gemini_to_html(&response, &url, is_proxy);
+  let mut html_context = String::from("<!DOCTYPE html><html><head>");
+  let gemini_html = gemini_to_html(&response, &url, is_proxy);
+  let gemini_title = gemini_html.0;
   let convert_time_taken = timer.elapsed();
 
   // Try to add an external stylesheet from the `CSS_EXTERNAL` environment
   // variable.
   if let Ok(css) = var("CSS_EXTERNAL") {
-    html_context.1.push_str(&format!(
+    html_context.push_str(&format!(
       "<link rel=\"stylesheet\" type=\"text/css\" href=\"{}\">",
       css
     ));
   }
 
+  // Try to add an external favicon from the `FAVICON_EXTERNAL` environment
+  // variable.
+  if let Ok(favicon) = var("FAVICON_EXTERNAL") {
+    html_context.push_str(&format!(
+      "<link rel=\"icon\" type=\"image/x-icon\" href=\"{}\">",
+      favicon
+    ));
+  }
+
   // Add a title to HTML response
-  html_context
-    .1
-    .push_str(&format!("<title>{}</title>", html_context.0));
+  html_context.push_str(&format!("<title>{}</title>", gemini_title));
+
+  html_context.push_str("</head><body>");
+
+  html_context.push_str(&gemini_html.1);
 
   // Add proxy information to footer of HTML response
-  html_context.1.push_str(&format!(
+  html_context.push_str(&format!(
     "<details>\n<summary>Proxy information</summary>
 <dl>
 <dt>Original URL</dt>
@@ -335,7 +360,7 @@ async fn default(req: actix_web::HttpRequest) -> Result<HttpResponse, Error> {
 </dl>
 <p>This content has been proxied by \
 <a href=\"https://github.com/gemrest/september{}\">September ({})</a>.</p>
-</details>",
+</details></body></html>",
     url,
     response.status,
     response.meta,
@@ -349,7 +374,7 @@ async fn default(req: actix_web::HttpRequest) -> Result<HttpResponse, Error> {
   Ok(
     HttpResponse::Ok()
       .content_type("text/html; charset=utf-8")
-      .body(html_context.1),
+      .body(html_context),
   )
 }
 
